﻿using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.DataAnalysis.Symbolic;

namespace HeuristicLab.EvolutionaryTracking {
  [Item("SymbolicExpressionTreeGenealogyGraph", "A genealogy graph for symbolic expression trees")]
  [StorableClass]
  public class SymbolicExpressionTreeGenealogyGraph : GenealogyGraph<ISymbolicExpressionTree> {

    [Storable]
    private readonly Dictionary<GenealogyGraphNode, NodeMetadata> _nodeInfo = new Dictionary<GenealogyGraphNode, NodeMetadata>();

    public SymbolicExpressionTreeGenealogyGraph() {
    }

    [StorableConstructor]
    public SymbolicExpressionTreeGenealogyGraph(bool serializing)
      : base(serializing) {
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new SymbolicExpressionTreeGenealogyGraph(this, cloner);
    }

    protected SymbolicExpressionTreeGenealogyGraph(SymbolicExpressionTreeGenealogyGraph original, Cloner cloner)
      : base(original, cloner) {
    }

    public override void AddNode(ISymbolicExpressionTree tree) {
      if (HasNode(tree)) return;
      var node = new GenealogyGraphNode(tree);
      _nodeInfo[node] = new NodeMetadata();
      base.AddNode(node);
    }

    public override void AddNode(GenealogyGraphNode node) {
      var tree = (ISymbolicExpressionTree)node.Data;
      if (HasNode(tree)) return;
      _nodeInfo[node] = new NodeMetadata();
      base.AddNode(node);
    }

    public NodeMetadata this[GenealogyGraphNode node] {
      get {
        NodeMetadata value;
        _nodeInfo.TryGetValue(node, out value);
        return value;
      }
      set {
        _nodeInfo[node] = value;
      }
    }

    #region Fragment tracing
    public IEnumerable<ISymbolicExpressionTree> TraceFragment(ISymbolicExpressionTreeNode fragment, int mode = 0) {
      return Keys.Where(tree => tree.ContainsFragment(fragment, mode));
    }
    #endregion
  }

  public class NodeMetadata {
    public double Quality { get; set; }
    public List<double> Ranks { get; set; }
    public bool IsElite { get; set; }

    public NodeMetadata() {
      Ranks = new List<double>();
    }
  }
}
